<?php

namespace App\Http\Controllers;


use App\Models\Address;
use App\Models\Client;
use App\Models\Orders;
use App\Models\OrderService;
use App\Models\Payment;
use App\Models\Product;
use App\Models\Service;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use PayPalCheckoutSdk\Orders\OrdersGetRequest;
use Sample\GetOrder;
use Sample\PayPalClient;

class OrderController extends Controller
{
    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        $this->middleware('auth');
    }

    /**
     * Show the application Clients page.
     *
     * @return \Illuminate\Contracts\Support\Renderable
     */
    public function index(Request $request)
    {

        $id = Auth::user()->id;
        $status_id = ($request->get('status_id')) ? $request->get('status_id') : 0;


        if ($status_id == 0) {


            $orders = Orders::select([
                'id',
                'client_id',
                'flight_id',
                'barcode',
                'operator_id',
                'name',
                'from_address',
                'boxes',
                'debet',
                'credit',
                'added',
                'edited',
                'total',
                'amount',
                'weight',
                'summa',
                'sign',
                'track_number',
                'is_site',
                'site_name',
                'site_track_number',
                'is_payment_uzb',
                'to_address_id',
                'payment_is_to_office',
                'client_type',
                'delivery_info',
                'payment_info',
                'status_id',
                'label_url',
                'comment',
                DB::raw('(SELECT name FROM payment_statuses WHERE id=(select payment_statuses_id from payment where order_id=orders.id)) as payment_status'),
                DB::raw('(SELECT title FROM flights WHERE id=orders.flight_id) as flights'),
                DB::raw('(SELECT name FROM users WHERE id=orders.operator_id) as operator'),
                DB::raw('(SELECT concat(name, \' \', surname) FROM clients WHERE id=orders.client_id) as client'),
                DB::raw('(SELECT title FROM status WHERE id=orders.status_id) as statuses'),
                DB::raw('(SELECT color_class FROM status WHERE id=orders.status_id) as status_class'),
                DB::raw('(SELECT concat(firstname,\' \',secondname,\' \',fathername) FROM address_info WHERE id=orders.to_address_id) as recipient'),
                DB::raw('(SELECT concat(country,\' \',city,\' \',street,\' \',house,\' \',apartment,\' \',zip) FROM address_info WHERE id=orders.to_address_id) as recipient_address'),
            ])->where('client_id', '=', $id)->get();

        } else {

            $orders = Orders::select([
                'id',
                'client_id',
                'flight_id',
                'barcode',
                'operator_id',
                'name',
                'from_address',
                'boxes',
                'debet',
                'credit',
                'added',
                'edited',
                'total',
                'amount',
                'weight',
                'summa',
                'sign',
                'track_number',
                'is_site',
                'site_name',
                'site_track_number',
                'is_payment_uzb',
                'to_address_id',
                'payment_is_to_office',
                'client_type',
                'delivery_info',
                'payment_info',
                'status_id',
                'label_url',
                'comment',
                DB::raw('(SELECT name FROM payment_statuses WHERE id=(select payment_statuses_id from payment where order_id=orders.id)) as payment_status'),
                DB::raw('(SELECT title FROM flights WHERE id=orders.flight_id) as flights'),
                DB::raw('(SELECT name FROM users WHERE id=orders.operator_id) as operator'),
                DB::raw('(SELECT concat(name, \' \', surname) FROM clients WHERE id=orders.client_id) as client'),
                DB::raw('(SELECT title FROM status WHERE id=orders.status_id) as statuses'),
                DB::raw('(SELECT color_class FROM status WHERE id=orders.status_id) as status_class'),
                DB::raw('(SELECT concat(firstname,\' \',secondname,\' \',fathername) FROM address_info WHERE id=orders.to_address_id) as recipient'),
                DB::raw('(SELECT concat(country,\' \',city,\' \',street,\' \',house,\' \',apartment,\' \',zip) FROM address_info WHERE id=orders.to_address_id) as recipient_address'),
            ])->where([
                ['client_id', '=', $id],
                ['status_id', '=', $status_id]
            ])->get();


        }

        $statuses = DB::select('SELECT s.id, s.sort, s.title, s.color_class, (SELECT count(*) FROM orders where status_id=s.id and client_id=' . $id . ') as count FROM status s');

        $count = DB::select('SELECT COUNT(*) as count FROM orders where client_id=' . $id);


//        dd($orders);
        return view('pages.parcels.allpackages', ['orders' => $orders, 'count' => $count[0]->count, 'status_id' => $status_id, 'statuses' => $statuses]);
    }


    /**
     * Show the application Client create page.
     *
     * @return \Illuminate\Contracts\Support\Renderable
     */
    public function create($id)
    {

        $client = Client::find($id);

        $senderAddresses = Address::select([
            'id',
            'user_id',
            'country',
            'city', 'street', 'house', 'apartment', 'zip', 'passport', 'type'
        ])->where('user_id', '=', $id)->get();
//        $senderAddresses->where('user_id', '=', $id)->get();
        $recAddresses = Address::select([
            'id',
            'country',
            'city', 'street', 'house', 'apartment', 'zip', 'passport', 'type'
        ])->where('user_id', '=', $id)->get();
//        $recAddresses->where('user_id', '=', $id)->get();
        $services = Service::select([
            'id', 'name_ru', 'price', 'status'
        ])->where('status', '=', 1)->get();


        return view('pages.parcels.order_create', ['client' => $client, 'senderAddresses' => $senderAddresses, 'recAddresses' => $recAddresses, 'services' => $services]);
    }

    public function payment($orderId)
    {
        $order = Orders::select([
            'id',
            'client_id',
            'flight_id',
            'barcode',
            'operator_id',
            'name',
            'from_address',
            'boxes',
            'debet',
            'credit',
            'added',
            'edited',
            'total',
            'amount',
            'weight',
            'summa',
            'sign',
            'track_number',
            'is_site',
            'site_name',
            'site_track_number',
            'is_payment_uzb',
            'to_address_id',
            'payment_is_to_office',
            'client_type',
            'delivery_info',
            'payment_info',
            'status_id',
            'label_url',
            'comment',
            DB::raw('(SELECT title FROM flights WHERE id=orders.flight_id) as flights'),
            DB::raw('(SELECT name FROM users WHERE id=orders.operator_id) as operator'),
            DB::raw('(SELECT concat(name, \' \', surname) FROM clients WHERE id=orders.client_id) as client'),
            DB::raw('(SELECT title FROM status WHERE id=orders.status_id) as statuses'),
            DB::raw('(SELECT color_class FROM status WHERE id=orders.status_id) as status_class'),
            DB::raw('(SELECT concat(firstname,\' \',secondname,\' \',fathername) FROM address_info WHERE id=orders.from_address) as sender'),
            DB::raw('(SELECT concat(firstname,\' \',secondname,\' \',fathername) FROM address_info WHERE id=orders.to_address_id) as recipient'),
            DB::raw('(SELECT concat(passport,\' | \',passport_by,\', \',passport_issue) FROM address_info WHERE id=orders.to_address_id) as recipient_passport'),
            DB::raw('(SELECT concat(country,\', \',city,\', \',street,\' \',house,\' \',apartment,\', \',zip) FROM address_info WHERE id=orders.to_address_id) as recipient_address'),
        ])->where([
            ['id', '=', $orderId]
        ])->get();

//        dd($order);

        $payment = Payment::select([
            'id',
            'transaction_number',
            'order_id',
            'summa',
            'payment_statuses_id',
            'added_date',
            'payed_date',
            'description',
        ])->where('order_id', '=', $orderId)->get();

        return view('pages.parcels.payment', ['order' => $order[0], 'payment' => $payment[0]]);
    }

    public function paymentSuccess($orderId){

//        dd($orderId);

        $client = PayPalClient::client();
        $response = $client->execute(new OrdersGetRequest($orderId));

        /**
         *Enable the following line to print complete response as JSON.
         */
        //print json_encode($response->result);
        print "Status Code: {$response->statusCode}\n";
        print "Status: {$response->result->status}\n";
        print "Order ID: {$response->result->id}\n";
        print "Intent: {$response->result->intent}\n";
        print "Links:\n";
        foreach($response->result->links as $link)
        {
            print "\t{$link->rel}: {$link->href}\tCall Type: {$link->method}\n";
        }
        // 4. Save the transaction in your database. Implement logic to save transaction to your database for future reference.
        print "Gross Amount: {$response->result->purchase_units[0]->amount->currency_code} {$response->result->purchase_units[0]->amount->value}\n";

        if (!count(debug_backtrace()))
        {
            GetOrder::getOrder($response->result->id);
        }
    }

    public function orderStore(Request $request)
    {
        $user = Auth::user();

        $orderServices = $request->get('services_id');
        $orderProducts = $request->get('products');
        $price = 0;
        $weight = 0;
        $amount = 0;
        $serSum = 0;

        if ($orderProducts != null) {
            foreach ($orderProducts as $item) {
                $price += $item[2];
//                $weight += $item[2];
                $amount += $item[1];
            }
        }

        if ($orderServices != null) {

            foreach ($orderServices as $item) {
                $ser = Service::find($item);
                $serSum += $ser->price;
            }
        }


        $order = Orders::create([
            'client_id' => $user->id,
            'name' => $request->get('name'),
            'flight_id' => 0,
            'added' => Carbon::now(),
            'amount' => $amount,
            'summa' => ($request->get('weight') * 8.5) + $serSum,
            'weight' => $request->get('weight'),
            'total' => $price,
            'track_number' => '98' . $this->generateUniqNumber(4),
            'is_site' => $request->get('is_site'),
            'is_payment_uzb' => 0,
            'from_address' => $request->get('from_address'),
            'to_address_id' => $request->get('to_address_id'),
            'payment_is_to_office' => 0,
            'delivery_info' => $request->get('delivery_info'),
            'payment_info' => $request->get('payment_type_id'),
            'comment' => $request->get('comment'),
            'client_type' => 'WEB',
            'status_id' => 19,
        ]);

        if ($orderServices != null) {

            foreach ($orderServices as $item) {
                OrderService::create([
                    'orders_id' => $order->id,
                    'services_id' => $item
                ]);
            }
        }

        if ($orderProducts != null) {
            foreach ($orderProducts as $item) {
                Product::create([
                    'title' => $item[0],
                    'comment' => $item[3],
                    'price' => $item[2],
                    'amount' => $item[1],
                    'order_id' => $order->id,
                ]);
            }
        }

        $payStatusId = 0;
        if ($request->get('payment_type_id') == 1) {
            $payStatusId = 1;
        } elseif ($request->get('payment_type_id') == 2) {
            $payStatusId = 5;
        } elseif ($request->get('payment_type_id') == 3) {
            $payStatusId = 6;
        }


        Payment::create([
            'transaction_number' => $this->generateUniqNumber(12),
            'order_id' => $order->id,
            'summa' => $order->summa,
            'payment_statuses_id' => $payStatusId,
            'added_date' => Carbon::now()
        ]);

        return redirect(route('package'));
    }

    public function getAddressAjaxS()
    {
        $senderAddresses = Address::select([
            'id',
            'user_id',
            'country',
            'firstname', 'secondname','phone',
            'city', 'street', 'house', 'apartment', 'zip', 'passport', 'type',
            DB::raw('(SELECT title_ru FROM country WHERE id=address_info.country) as country_name'),
            DB::raw('(SELECT title_ru FROM city WHERE id=address_info.city) as city_name'),
        ])->where([
            ['user_id', '=', Auth::id()],
            ['type', '=', 'sender'],
        ])->get();
//        $senderAddresses->where('user_id', '=', $id)->get();

        return json_decode($senderAddresses);
    }

    public function getAddressAjaxR()
    {
        $recAddresses = Address::select([
            'id',
            'user_id',
            'country',
            'firstname', 'secondname',
            'city', 'street', 'house', 'apartment', 'zip', 'passport', 'type',
            DB::raw('(SELECT title_ru FROM country WHERE id=address_info.country) as country_name'),
            DB::raw('(SELECT title_ru FROM city WHERE id=address_info.city) as city_name'),
        ])->where([
            ['user_id', '=', Auth::id()],
            ['type', '=', 'recipient'],
        ])->get();

        return json_decode($recAddresses);
    }

}
